/* dptsv.h - C language wrapper for the LAPACK function DPTSV
 *
 * Copyright (C) 2005  Jochen Voss.
 *
 * $Id: dptsv.h 6703 2005-11-05 22:27:55Z voss $
 *
 * Purpose
 * =======
 *
 * DPTSV computes the solution to a real system of linear equations
 * A*X = B, where A is an N-by-N symmetric positive definite tridiagonal
 * matrix, and X and B are N-by-NRHS matrices.
 *
 * A is factored as A = L*D*L**T, and the factored form of A is then
 * used to solve the system of equations.
 *
 * Arguments
 * =========
 *
 * N       (input) INTEGER
 *         The order of the matrix A.  N >= 0.
 *
 * NRHS    (input) INTEGER
 *         The number of right hand sides, i.e., the number of columns
 *         of the matrix B.  NRHS >= 0.
 *
 * D       (input/output) DOUBLE PRECISION array, dimension (N)
 *         On entry, the n diagonal elements of the tridiagonal matrix
 *         A.  On exit, the n diagonal elements of the diagonal matrix
 *         D from the factorization A = L*D*L**T.
 *
 * E       (input/output) DOUBLE PRECISION array, dimension (N-1)
 *         On entry, the (n-1) subdiagonal elements of the tridiagonal
 *         matrix A.  On exit, the (n-1) subdiagonal elements of the
 *         unit bidiagonal factor L from the L*D*L**T factorization of
 *         A.  (E can also be regarded as the superdiagonal of the unit
 *         bidiagonal factor U from the U**T*D*U factorization of A.)
 *
 * B       (input/output) DOUBLE PRECISION array, dimension (LDB,NRHS)
 *         On entry, the N-by-NRHS right hand side matrix B.
 *         On exit, if INFO = 0, the N-by-NRHS solution matrix X.
 *
 * LDB     (input) INTEGER
 *         The leading dimension of the array B.  LDB >= max(1,N).
 *
 * INFO    (output) INTEGER
 *         = 0:  successful exit
 *         < 0:  if INFO = -i, the i-th argument had an illegal value
 *         > 0:  if INFO = i, the leading minor of order i is not
 */

#ifndef FILE_DPTSV_H_SEEN
#define FILE_DPTSV_H_SEEN

#ifdef __APPLE__
#include <veclib/veclib.h>
#else
extern  void  dptsv_ (long *Np, long *NRHSp, double *D, double *E,
                      double *B, long *LDBp, long *INFOp);
#endif

static inline long
dptsv (long n, long NRHS, double *D, double *E, double *B, long ldb)
{
  long info;
  dptsv_ (&n, &NRHS, D, E, B, &ldb, &info);
  return info;
}

#endif /* FILE_DPTSV_H_SEEN */
