/* stoheat.c - numerical solution of a stochastic heat equation
 *
 * Copyright (C) 2005  Jochen Voss
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 * $Id: stoheat.c 6703 2005-11-05 22:27:55Z voss $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "jvrandom.h"
#include "dptsv.h"

/* Dirichlet boundary conditions */
#define  U0  -2.0
#define  U1  -1.0

/* Discretisation parameters */
#define  N  100
#define  DX  (1.0/N)
#define  DT  (DX*DX)

#define  TMAX  2.0

/* mean of the non-homogeneous problem */
static  double  m[N+1];

/* current state of the SPDE (full solution is U+m) */
static  double  U[N+1];

/* diagonal of the triband matrix on the left-hand side */
static  double  L0[N-1];

/* subdiagonal of the triband matrix on the left-hand side */
static  double  L1[N-2];


static double
f (double u)
/* the drift of the SPDE */
{
  return  -5*u;
}

int
main ()
{
  double  delta, sigma;
  int  i, k;

  init_noise (time (NULL));

  /* an abreviation for the triband matrix on the right-hand side */
  delta = DT/(2*DX*DX);
  /* the standard deviation of the noise */
  sigma = sqrt (DT/DX);
  
  /* Calculate the "mean": we assume Dirichlet boundary conditions if
   * U0 (left) or U1 (right) are set, and homogeneous v. Neumann
   * boundary conditions otherwise.  */
#if defined(U0) && defined(U1)
  for (i=0; i<=N; ++i) {
    double  q = 1.0/N;
    m[i] = i*q*U1 + (N-i)*q*U0;
  }
#elif defined(U0)
  for (i=0; i<=N; ++i)  m[i] = U0;
#elif defined(U1)
  for (i=0; i<=N; ++i)  m[i] = U1;
#else
  for (i=0; i<=N; ++i)  m[i] = 0;
#endif

  /* Prepare the tridiagonal matrix for the left-hand side of the
   * evolution equation.  We store the data in a convenient form for
   * the DPTSV call below.  */
#if defined(U0)
  L0[0] = 1 + 2*DT/(2*DX*DX);
#else
  L0[0] = 1 + 1*DT/(2*DX*DX);
#endif
  for (i=1; i<N-2; ++i)  L0[i] = 1 + 2*DT/(2*DX*DX);
#if defined(U1)
  L0[N-2] = 1 + 2*DT/(2*DX*DX);
#else
  L0[N-2] = 1 + 1*DT/(2*DX*DX);
#endif
  for (i=0; i<N-2; ++i)  L1[i] = - 1*DT/(2*DX*DX);

  /* initial condition: we choose u_0 = m, i.e. U=0 */
  for (i=0; i<N-1; ++i)  U[i] = 0;

  for (k=0; k*DT<TMAX+0.5*DT; ++k) {
    double  tmp[N+1];

    /* emit data for gnuplot */
    if (k % (int)(0.1/DT+0.5) == 0) {
      for (i=0; i<=N; ++i)  printf ("%f %f %f\n", i*DX, k*DT, U[i]+m[i]);
      putchar ('\n');
    }

    /* calculate the right hand side */
    for (i=1; i<N; ++i) {
      double  a, b, c;

      if (i==1) {
#if defined(U0)
	a = (1-2*delta)*U[i] + delta*U[i+1];
#else
	a = (1-delta)*U[i] + delta*U[i+1];
#endif
      } else if (i==N-1) {
#if defined(U1)
	a = delta*U[i+1] + (1-2*delta)*U[i];
#else
	a = delta*U[i+1] + (1-delta)*U[i];
#endif
      } else {
	a = delta*U[i-1] + (1-2*delta)*U[i] + delta*U[i+1];
      }
      b = f (U[i]+m[i]) * DT;
      c = sigma * jvrand_standard_normal ();
      tmp[i] = a + b + c;
    }

    /* solve the tridiagonal system */
    {
      double  E0[N-1], E1[N-2];

      memcpy (E0, L0, sizeof(E0));
      memcpy (E1, L1, sizeof(E1));
      dptsv (N-1, 1, E0, E1, tmp+1, N-1);
    }

    memcpy (U+1, tmp+1, (N-1)*sizeof(double));
#if defined(U0)
    U[0] = 0;
#else
    U[0] = U[1];
#endif
#if defined(U1)
    U[N] = 0;
#else
    U[N] = U[N-1];
#endif
  }
  
  return  0;
}
